package main

import (
	"encoding/json"
	"fmt"
	"os"
	"time"

	"git.ctrlz.es/mgdelacroix/bookworm/templates"

	"github.com/alecthomas/kong"
	miniflux "miniflux.app/client"
)

type Config struct {
	MinifluxURL      string `json:"miniflux_url"`
	MinifluxUsername string `json:"miniflux_username"`
	MinifluxPassword string `json:"miniflux_password"`
	MinifluxAPI      string `json:"miniflux_api"`
}

func (c *Config) IsValid() error {
	if c.MinifluxURL == "" {
		return fmt.Errorf("miniflux_url is required")
	}

	if c.MinifluxAPI == "" && (c.MinifluxUsername == "" || c.MinifluxPassword == "") {
		return fmt.Errorf("either miniflux_api or miniflux_username and miniflux_password need to be set")
	}

	return nil
}

func ReadConfig(name string) (*Config, error) {
	b, err := os.ReadFile(name)
	if err != nil {
		return nil, err
	}

	var cfg Config
	if err := json.Unmarshal(b, &cfg); err != nil {
		return nil, err
	}

	return &cfg, nil
}

func createClient(cfg *Config) (*miniflux.Client, error) {
	var client *miniflux.Client
	if cfg.MinifluxAPI != "" {
		client = miniflux.New(cfg.MinifluxURL, cfg.MinifluxAPI)
	} else {
		client = miniflux.New(cfg.MinifluxURL, cfg.MinifluxUsername, cfg.MinifluxPassword)
	}

	if _, err := client.Me(); err != nil {
		return nil, fmt.Errorf("cannot get \"me\" with newly created client: %w", err)
	}

	return client, nil
}

func checkErr(err error) {
	if err != nil {
		fmt.Fprintf(os.Stderr, "error: %s\n", err)
		os.Exit(1)
	}
}

func listCategories(client *miniflux.Client) error {
	categories, err := client.Categories()
	if err != nil {
		return fmt.Errorf("cannot fetch categories: %w", err)
	}

	if len(categories) != 0 {
		fmt.Println("Category list")
	}
	for _, c := range categories {
		fmt.Printf("- %s\n", c.Title)
	}

	return nil
}

func (l *listCmd) Run(ctx *Context) error {
	cfg, err := ReadConfig(ctx.Config)
	checkErr(err)

	cErr := cfg.IsValid()
	checkErr(cErr)

	client, cliErr := createClient(cfg)
	checkErr(cliErr)

	return listCategories(client)
}

func (g *generateCmd) Run(ctx *Context) error {
	cfg, err := ReadConfig(ctx.Config)
	checkErr(err)

	cErr := cfg.IsValid()
	checkErr(cErr)

	client, cliErr := createClient(cfg)
	checkErr(cliErr)

	var feeds miniflux.Feeds
	var fErr error
	if g.Category != "" {
		categories, catErr := client.Categories()
		if catErr != nil {
			checkErr(fmt.Errorf("can't get categories: %w", catErr))
		}

		var category *miniflux.Category
		for _, c := range categories {
			if c.Title == g.Category {
				category = c
				break
			}
		}

		if category == nil {
			checkErr(fmt.Errorf("can't find category %q", g.Category))
		}

		feeds, fErr = client.CategoryFeeds(category.ID)
	} else {
		feeds, fErr = client.Feeds()
	}
	checkErr(fErr)

	baseTitle := "All"
	if g.Title != "" {
		baseTitle = g.Title
	} else if g.Category != "" {
		baseTitle = g.Category
	}

	title := fmt.Sprintf(
		"%s %s %s",
		baseTitle,
		time.Now().Add(time.Hour*24*-7).Format("01-02"),
		time.Now().Format("01-02"),
	)

	data := struct {
		Title              string
		OldestArticle      int
		MaxArticlesPerFeed int
		Feeds              miniflux.Feeds
	}{
		Title:              title,
		OldestArticle:      7,
		MaxArticlesPerFeed: 100,
		Feeds:              feeds,
	}

	s, tErr := templates.Execute(data)
	checkErr(tErr)

	checkErr(os.WriteFile("out.recipe", s, 0755))

	// ToDo: call ebook-convert with the right filename and the extension from the format
	// ToDo: make recipe a temp file that is removed after the ebook is generated

	return nil
}

func main() {
	ctx := kong.Parse(&cli)
	if err := ctx.Run(&Context{Config: cli.Config}); err != nil {
		fmt.Fprintf(os.Stderr, "error: %s\n", err)
		os.Exit(1)
	}
}
